import {UNIT_PAY, IS_PAY, MY_BALANCE, PAY_CHECK_INTERVAL, GET_EXCLUDE_PAYTYPE} from "./API";
import md5 from 'blueimp-md5'
import qs from 'qs'
import {Utils} from "@/Utils/Utils";
import Decimal from 'decimal.js';
import {defaultRequest, DefaultResponse} from "@/Request/Request";
import {
    BACK_ORDER_ID,
    BackInfo,
    DoUnifiedParams,
    FULL_PAY_METHODS,
    OrderStateCallBack,
    PayChannel,
    PayError,
    PayErrorMessage,
    PaymentParams,
    PayMethod,
    PayType,
    ToPayParams,
    ToPayReturns,
    WechatPayParams
} from "./Defined";

export * from './Defined'

const requestForJava = defaultRequest

export class Payment {
    balance = 0;
    limitPayChannels: PayChannel[] = [];
    isEnabledCombinationPay: boolean = true; // 配置的是否开启组合支付
    supportCombination: boolean = true; // 余额开关的改变是否支持组合支付
    constructor(paymentParams?: PaymentParams) {
        this.limitPayChannels = paymentParams?.payChannels ?? Object.keys(PayChannel).map(key => key) as PayChannel[];
        if (typeof paymentParams?.supportCombination === 'boolean') {
            this.supportCombination = typeof paymentParams?.supportCombination === 'boolean' && paymentParams.supportCombination
        }

        if (typeof paymentParams?.isEnabledCombinationPay === 'boolean') {
            this.isEnabledCombinationPay = typeof paymentParams?.isEnabledCombinationPay === 'boolean' && paymentParams.isEnabledCombinationPay
        }
    }

    // 原型方法设置是否支持组合支付
    setIsSupportCombination (value: boolean) {
        this.supportCombination = value
    }

    // 获取余额
    getBalance() {
        return new Promise<number|null>(async (resolve, reject) =>{
            try {
                const res = await requestForJava.get<Record<string, string>, DefaultResponse>(MY_BALANCE)
                console.log("🚀 ~ file: Payment.ts:52 ~ Payment ~ returnnewPromise<number|null> ~ res:", res)
                if (res.code === '200') {
                    this.balance = Number(res.data)
                    resolve(this.balance)
                } else {
                    reject(res)
                }
            } catch (error) {
                reject(error)
            }
        })
    }

    // 获取被禁止的支付方式列表
    getDisabledPayMethodList (doctorId: string | undefined, orderId?: string | undefined):Promise<number[]> {
        return new Promise<number[]>(async (resolve, reject) => {
            try {
                const res:any = await requestForJava.get<Record<string, string>, DefaultResponse>(GET_EXCLUDE_PAYTYPE, {
                    params: {
                        doctorId,
                        orderId
                    }
                })
                if (res.code == 200) {
                    resolve(res.data)
                } else {
                    reject()
                }
            } catch (error) {
                reject(error)
            }
        })
    }

    // 获取支付方式列表
    getPayMethodList (totalAmount:number, doctorId?:string, orderId?:string) {
        // 计算支付方式列表
        const List = this.supportCombination ? this.getPayMethodListForCombination(totalAmount) : this.getPayMethodListForNotCombination(totalAmount)

        return  new Promise(async (resolve, reject) => {
            try {
                const res = await this.getDisabledPayMethodList(doctorId, orderId)
                console.log("🚀 ~ file: Payment.ts:93 ~ Payment ~ returnnewPromise ~ res:", res)
                // 从list中过滤出被禁止的支付方式
                const filterList = List.filter(({value}) => !res.includes(value))
                console.log("🚀 ~ file: Payment.ts:95 ~ Payment ~ returnnewPromise ~ filterList:", filterList)
                resolve(filterList)
            } catch (error) {
                console.log("🚀 ~ file: Payment.ts:98 ~ Payment ~ returnnewPromise ~ error:", error)
                reject(error)
            }
        })
    }

    // 不支持组合支付的情况下获取支付方式列表
    private getPayMethodListForNotCombination(totalAmount: number): PayMethod[] {
        return FULL_PAY_METHODS.filter(({value}) => {
            switch (value) {
                case PayType.BALANCE:
                    return this.balance > 0
                case PayType.WECHAT:
                    return !Utils.isAlipay()
                case PayType.ALIPAY:
                    return !Utils.isWechat()
                case PayType.HUABEI:
                    return !Utils.isWechat()
            }
        }).map(item => {
            let caclCombination = false
            if (this.isEnabledCombinationPay) {
                caclCombination = this.supportCombination ? (this.balance > 0 && this.balance < totalAmount) : true
            } else {
                caclCombination =  this.balance < totalAmount 
            }

            switch (item.value) {
                case PayType.BALANCE:
                    return {
                        ...item,
                        label: `余额<span class="balance">（可用¥${this.balance}）</span>`,
                        disabled:  caclCombination,
                    }
            }
            return item
        })
    }

    // 支持组合支付的情况下获取支付方式列表
    private getPayMethodListForCombination(totalAmount: number): PayMethod[] {
        return FULL_PAY_METHODS.filter(({value}) => {
            switch (value) {
                case PayType.BALANCE:
                    return this.balance > 0
                case PayType.WECHAT:
                    return this.balance === 0 && !Utils.isAlipay()
                case PayType.WECHAT_BALANCE:
                    return this.balance > 0 && this.balance < totalAmount && !Utils.isAlipay()
                case PayType.ALIPAY:
                    return this.balance === 0 && !Utils.isWechat()
                case PayType.ALIPAY_BALANCE:
                    return this.balance > 0 && this.balance < totalAmount && !Utils.isWechat()
                case PayType.HUABEI:
                    return this.balance === 0 && !Utils.isWechat()
                case PayType.HUABEI_BALANCE:
                    return this.balance > 0 && this.balance < totalAmount && !Utils.isWechat()
            }
        }).map(item => {
            switch (item.value) {
                case PayType.BALANCE:
                    return {
                        ...item,
                        label: `余额<span class="balance">（可用¥${this.balance}）</span>`,
                        disabled: this.balance > 0 && this.balance < totalAmount
                    }
            }
            return item
        })
    }

    // 计算非组合支付金额
    computeAmount(totalAmount: number, payType: PayType): { payAmount: number, payBalance: number } {
        let payBalance = 0, payAmount = 0
        if(payType == PayType.BALANCE){
            payBalance = totalAmount;
            payAmount = 0;
        }
        else{
            payBalance = 0;
            payAmount = totalAmount;
        }
        return {payAmount, payBalance}
    }

    // 计算组合支付金额
    computeAmountForCombination(totalAmount: number, balance: number = 0): { payAmount: number, payBalance: number } {
        console.log("🚀 ~ file: Payment.ts:171 ~ Payment ~ computeAmountForCombination ~ balance:", balance)
        let payBalance = 0, payAmount = 0
        if (balance === 0) {
            payAmount = totalAmount
        } else if (balance > 0 && balance < totalAmount) {
            payBalance = balance;
            const x = new Decimal(totalAmount);
            const y = new Decimal(balance); 
            payAmount = Number(x.minus(y));
        } else {
            payBalance = totalAmount
        }
        return {payAmount, payBalance}
    } 

    // 去支付
    async toPay(params: ToPayParams): Promise<ToPayReturns> {
        console.log("🚀 ~ file: Payment.ts:179 ~ Payment ~ toPay ~ params:", params)
        const {totalAmount, payType, returnUrl, orderId, subOrderIds, redirectUrl} = params
        let quitUrl = params.quitUrl
        // validate start >>>
        switch (payType) {
            case PayType.BALANCE:
                if (this.balance < totalAmount) {
                    return {errorMessage: PayErrorMessage.BALANCE_EMPTY, success: false, errorType: PayError.VALIDATE}
                }
                break
            case PayType.WECHAT_BALANCE:
            case PayType.WECHAT:
                if (this.limitPayChannels.includes(PayChannel.WX_MWEB) && this.limitPayChannels.includes(PayChannel.WX_JSAPI)) {
                    if (!redirectUrl) {
                        return {
                            errorMessage: PayErrorMessage.WECHAT_REDIRECT_URL_MISSING,
                            success: false,
                            errorType: PayError.VALIDATE
                        }
                    }
                } else {
                    return {
                        errorMessage: PayErrorMessage.WECHAT_PAY_CHANNEL_MISSING,
                        success: false,
                        errorType: PayError.VALIDATE
                    }
                }
                break
            case PayType.ALIPAY:
            case PayType.ALIPAY_BALANCE:
            case PayType.HUABEI:
            case PayType.HUABEI_BALANCE:
            case PayType.HUABEI_STAGE:
            case PayType.HUABEI_STAGE_BALANCE:
                if (this.limitPayChannels.includes(PayChannel.ALI_WAP)) {
                    if (!quitUrl) {
                        return {
                            errorMessage: PayErrorMessage.ALIPAY_QUIT_URL_MISSING,
                            success: false,
                            errorType: PayError.VALIDATE
                        }
                    } else if (!returnUrl) {
                        return {
                            errorMessage: PayErrorMessage.ALIPAY_RETURN_URL_MISSING,
                            success: false,
                            errorType: PayError.VALIDATE
                        }
                    }
                } else {
                    return {
                        errorMessage: PayErrorMessage.ALIPAY_PAY_CHANNEL_MISSING,
                        success: false,
                        errorType: PayError.VALIDATE
                    }
                }
                break
        }
        // validate end <<<
        // hack, 后端微信redirectUrl用的quitUrl
        const payChannel = this.getPayChannel(payType)
        if (payChannel === PayChannel.WX_MWEB || payChannel === PayChannel.WX_JSAPI) {
            quitUrl = redirectUrl
        }
        const { payAmount, payBalance } = this.supportCombination ? this.computeAmountForCombination(totalAmount, this.balance) : this.computeAmount(totalAmount, payType);
        const doUnifiedParams: DoUnifiedParams = {
            returnUrl,
            quitUrl,
            orderId,
            payType,
            payAmount, 
            payBalance,
            subOrderIds
        }
        if (payChannel !== null) {
            doUnifiedParams.payChannel = payChannel
        }
        const res = await requestForJava.post<DoUnifiedParams, DefaultResponse>(UNIT_PAY, doUnifiedParams)
        if (Utils.isWechat() && res.msg === PayErrorMessage.WECHAT_OPENID_APPID_NOT_MATCH) {
            window.location.href = `https://m.ydl.com/pay/wx-pay-open-id?toLink=${window.location.href}`;
        }
        if (res.code !== '200') return {errorMessage: res.msg as any, success: false, errorType: PayError.BACKEND}
        if (params.payType === PayType.BALANCE && res.data) {
            return {success: res.data.result, errorType: PayError.BALANCE}
        } else if (payChannel === PayChannel.WX_JSAPI) {
            const isPaySucc = await this.wechatPayJSSDK(res.data.content)
            return {
                errorMessage: isPaySucc ? undefined : PayErrorMessage.WECHAT_JSSDK_PAY_ERROR,
                success: isPaySucc,
                errorType: PayError.WECHAT_JSSDK
            }
        } else if (payChannel === PayChannel.WX_MWEB) {
            return {
                payUrl: res.data.content.link,
                success: false,
                errorType: PayError.WECHAT_H5_BREAK,
                errorMessage: PayErrorMessage.WECHAT_H5_PAY_BREAK
            }
        } else if (payChannel === PayChannel.ALI_WAP) {
            return {
                payUrl: res.data.content.link,
                success: false,
                errorType: PayError.ALIPAY_H5_BREAK,
                errorMessage: PayErrorMessage.ALIPAY_H5_PAY_BREAK
            }
        } else if (payChannel === PayChannel.PAYPAY) {
            
            return {
                payPalOrderId: res.data.content.payPalOrderId,
                errorMessage: res.msg as any
            }
        }
        return {success: false, errorType: PayError.UNKNOWN}
    }

    private getPayChannel(payType: PayType): PayChannel | null {
        switch (payType) {
            case PayType.PAYPAY:
                return PayChannel.PAYPAY
            case PayType.WECHAT_BALANCE:
            case PayType.WECHAT:
                return Utils.isWechat() ? PayChannel.WX_JSAPI : PayChannel.WX_MWEB
            case PayType.ALIPAY:
            case PayType.ALIPAY_BALANCE:
            case PayType.HUABEI:
            case PayType.HUABEI_BALANCE:
            case PayType.HUABEI_STAGE:
            case PayType.HUABEI_STAGE_BALANCE:
                return PayChannel.ALI_WAP
        }
        return null
    }

    private wechatPayJSSDK(wechatPayParams: WechatPayParams): Promise<boolean> {
        const {appId, timeStamp, nonceStr, signType, paySign, package: packageAlias} = wechatPayParams;

        return new Promise((resolve) => {
            window.WeixinJSBridge.invoke(
                "getBrandWCPayRequest",
                {
                    appId: appId, //公众号名称，由商户传入
                    timeStamp: timeStamp, //时间戳，自1970年以来的秒数
                    nonceStr: nonceStr, //随机串
                    package: packageAlias,
                    signType: signType, //微信签名方式：
                    paySign: paySign //微信签名
                },
                (res: any) => {
                    if (res.err_msg === "get_brand_wcpay_request:ok") {
                        resolve(true)
                    } else {
                        resolve(false)
                        console.error(res)
                    }
                }
            );
        })
    }

    // 支付结果轮询
    static async loopValidateOrderState(count = 0, callBack: OrderStateCallBack, pollTime = 3000): Promise<void> {
        const [, queryString] = window.location.href.split('?')
        const query = queryString ? qs.parse(queryString, {ignoreQueryPrefix: true}) : {}
        const returnParams: BackInfo = {orderId: Number(query[BACK_ORDER_ID])}
        let timer: any = null;
        const checkLoop = () => {
            if (--count > 0) {
                timer = setTimeout(() => {
                    this.loopValidateOrderState(count, callBack, pollTime)
                }, pollTime)
            } else {
                callBack({isPay: false, ...returnParams, isLimit:true})
                timer && clearTimeout(timer)
            }
        }
        if (query[BACK_ORDER_ID]) {
            try {
                const res = await requestForJava.post<Record<string, string>, DefaultResponse>(IS_PAY, {
                    orderId: query[BACK_ORDER_ID]
                })
                if (res.code === '200' && res.data === true) {
                    callBack({isPay: true, ...returnParams})
                    timer && clearTimeout(timer)
                } else {
                    checkLoop()
                }
            } catch (err) {
                checkLoop()
            }
        }
    }

    /**
     * 创建订单查询页面链接， 返回的url不做encode处理
     * @param url
     * @param backOrderId
     */
    static createBackInfoUrl(url: string, backOrderId: number): string {
        const [path, queryString] = url.split('?')
        const query = queryString ? qs.parse(queryString, {ignoreQueryPrefix: true}) : {}
        query[BACK_ORDER_ID] = backOrderId?backOrderId.toString(): ''
        return `${path}?${qs.stringify(query)}`
    }

    static async PayCheckIntervaL(): Promise<number> {
        const res = await requestForJava.get<Record<string, string>, DefaultResponse>(PAY_CHECK_INTERVAL)
        let interval = 3000;
        if(res.code === '200'){
            interval = res.data * 1000;
            return interval
        }
        return interval;
    }
}